-- ============================================================================
-- == HTBrokerSupport - Provides support for integrating this addon into     ==
-- ==                   LibDataBroker (aka: LDB) supporting display addons.  ==
-- ============================================================================

HTBrokerSupport    = LibStub("AceAddon-3.0"):NewAddon("HTBrokerSupport", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local L            = LibStub("AceLocale-3.0"):GetLocale("HalonaTools")

local DropDownMenu = CreateFrame("Frame", "DropDownMenu")      -- Dropdown Menu
local info         = {}                                -- Dropdown Menu Options

local brokerObject = {}                        -- References the broker object.
local taskHandleHB = ""            -- Stores the handle for the heartbeat task.

-- ============================================================================
-- == OnInitialize()                                                         ==
-- ==                                                                        ==
-- == This function is called when the addon is initialised.                 ==
-- ============================================================================

function HTBrokerSupport:OnInitialize()

  self:CancelAllTimers()

end

-- ============================================================================
-- == OnEnable()                                                             ==
-- ==                                                                        ==
-- == This function is called when the addon is enabled.                     ==
-- ============================================================================

function HTBrokerSupport:OnEnable()

  HTBrokerSupport:CreateDataObject()

  HTBrokerSupport:AdjustHeartRate()

end

-- ============================================================================
-- == OnDisable()                                                            ==
-- ==                                                                        ==
-- == This function is called when the addon is disabled.                    ==
-- ============================================================================

function HTBrokerSupport:OnDisable()

  self:CancelAllTimers()

end

-- ============================================================================
-- == CreateDataObject()                                                     ==
-- ==                                                                        ==
-- == This function creates the LDB data object.                             ==
-- ============================================================================

function HTBrokerSupport:CreateDataObject()

  brokerObject = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("HalonaTools", {
    type = "launcher",
    label = L["addon-name"],
    OnClick = function(_, msg)
      HTBrokerSupport:OnButtonClick(msg)
    end,
    icon = "Interface\\AddOns\\HalonaTools\\Images\\icon",
    OnTooltipShow = function(tooltip)
      HTBrokerSupport:BuildTooltip(tooltip)
    end,
  })

  HTBrokerSupport:BuildTextLabel()

end

-- ============================================================================
-- == AdjustHeartRate()                                                      ==
-- ==                                                                        ==
-- == This function allows the rate at which the heartbeat occurs. Typically ==
-- == this function will be called after the player makes a configuration    ==
-- == change to either of the resting or combat interval pulse settings.     ==
-- ============================================================================

function HTBrokerSupport:AdjustHeartRate()

  -- Initially, we will not worry about changing the heartrate when entering
  -- and leaving combat. Should we wish to do so later, we need merely add
  -- calls to the PLAYER_REGEN_DISABLED/PLAYER_REGEN_ENABLED events that
  -- call this method after changing the rate-parameter.

  HTBrokerSupport:RestartHeart(HTConfig:GetBrokerSupportHeartbeatResting(nil))

end

-- ============================================================================
-- == RestartHeart(interval)                                                 ==
-- ==                                                                        ==
-- == This function starts/restarts the addon's heartbeat with the supplied  ==
-- == interval value. (interval = time-in-seconds.)                          ==
-- ============================================================================

function HTBrokerSupport:RestartHeart(interval)

  self:CancelAllTimers()

  taskHandleHB = self:ScheduleRepeatingTimer("DoHeartbeat", interval, interval)

end

-- ============================================================================
-- == DoHeartbeat(interval)                                                  ==
-- ==                                                                        ==
-- == This function performs all tasks that are required to be repeated on a ==
-- == periodic basis.                                                        ==
-- ============================================================================

function HTBrokerSupport:DoHeartbeat(interval)

  -- HTConsole:Debug("DoHeartbeat(" .. interval .. ") called.")

  HTBrokerSupport:BuildTextLabel()

end

-- ============================================================================
-- == OnButtonClick(buttonName)                                              ==
-- ==                                                                        ==
-- == This function processes clicks made by the user/player on the addon's  ==
-- == display object (typically the icon).                                   ==
-- ==                                                                        ==
-- == Options: "LeftButton", "RightButton".                                  ==
-- ============================================================================

function HTBrokerSupport:OnButtonClick(buttonName)

  if     buttonName == L["wow-api-lmb-click"] then
    HTBrokerSupport:OnLeftButtonClick()
  elseif buttonName == L["wow-api-rmb-click"] then
    HTBrokerSupport:OnRightButtonClick()
  end

end

-- ============================================================================
-- == OnLeftButtonClick()                                                    ==
-- ==                                                                        ==
-- == This function processes left mouse-button clicks made by the player on ==
-- == the addon's display object (typically the icon).                       ==
-- ==                                                                        ==
-- == Left             = Attempt to join the current guild raid event.       ==
-- == Left        +Alt = Show Busy Responder Messages.                       ==
-- == Left+Control     = Toggle Busy Responder.                              ==
-- == Left+Control+Alt = Clear Busy Responder Messages.                      ==
-- ============================================================================

function HTBrokerSupport:OnLeftButtonClick()

  if IsAltKeyDown() and IsControlKeyDown() then
    HTConsole:BusyResponderClearMessages()
    return
  elseif IsAltKeyDown() then
    HTConsole:BusyResponderShowMessages()
    return
  elseif IsControlKeyDown() then
    HTConfig:ToggleIsEnabledBusyResponder()
    HTBrokerSupport:DisplayStatus(L["ldb-menu-item-brenabled-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBusyResponder(nil)))
    HTBrokerSupport:BuildTooltip(tooltip)
    return
  else
    HalonaTools:JoinGuildRaidEvent()
    return
  end

end

-- ============================================================================
-- == OnRightButtonClick()                                                   ==
-- ==                                                                        ==
-- == This function processes right mouse-button clicks made by the player   ==
-- == on the addon's display object (typically the icon).                    ==
-- ==                                                                        ==
-- == Right             = Show Halonatools Menu.                             ==
-- == Right        +Alt = Perform Ready Check.                               ==
-- == Right+Control     = Open Config GUI.                                   ==
-- == Right+Control+Alt = Toggle BuffMonitor Combat-Start Checking.          ==
-- ============================================================================

function HTBrokerSupport:OnRightButtonClick()

  if IsAltKeyDown() and IsControlKeyDown() then
    HTConfig:ToggleIsEnabledBuffMonitorCheckOnCombatStart()
    HTBrokerSupport:DisplayStatus(L["ldb-menu-item-bm-on-combatcheck-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCheckOnCombatStart(nil)))
    return
  elseif IsAltKeyDown() then
    HTCommon:DoReadyCheck()
    return
  elseif IsControlKeyDown() then
    HTGUI:Config()
    return
  else
    HTBrokerSupport:ShowMenu()
    return
  end

end

-- ============================================================================
-- == BuildTextLabel()                                                       ==
-- ==                                                                        ==
-- == This function builds the text label which display addons may display.  ==
-- ============================================================================

function HTBrokerSupport:BuildTextLabel()

  local textLabel = ""

  textLabel = HTBrokerSupport:GetTextLabelContentBusyResponderMessageNumber(textLabel)
  textLabel = HTBrokerSupport:GetTextLabelContentGStatsMembers(textLabel)
  textLabel = HTBrokerSupport:GetTextLabelContentGStatsRaiders(textLabel)
  textLabel = HTBrokerSupport:GetTextLabelContentGStatsOfficers(textLabel)
  textLabel = HTBrokerSupport:GetTextLabelContentGStatsCharacters(textLabel)
  textLabel = HTBrokerSupport:GetTextLabelContentGuildPlayMode(textLabel)
  textLabel = HTBrokerSupport:GetTextLabelContentCombatStatus(textLabel)
  textLabel = HTBrokerSupport:GetTextLabelContentAddonName(textLabel)

  brokerObject.text = textLabel

end

-- ============================================================================
-- == GetTextLabelContentBusyResponderMessageNumber(textLabel)               ==
-- ==                                                                        ==
-- == This function adds the number of currently stored BusyResponder        ==
-- == messages to the text label.                                            ==
-- ==                                                                        ==
-- == Example Label Addition: BR(12)                                         ==
-- ==                                                                        ==
-- == Colourisation: If BR enabled, BR is green, otherwise BR is red.        ==
-- ==                If num-msgs > 0, num-msgs is green, otherwise white.    ==
-- ============================================================================

function HTBrokerSupport:GetTextLabelContentBusyResponderMessageNumber(textLabel)

  if HTConfig:GetIsEnabledTextLabelContentBRMessageNum(nil) == true then

    local numMessages     = HTConsole:BusyResponderGetNumberOfMessagesStored()
    local brMessageText   = L["ldb-textlabel-busyresponder"]

    -- Set the text color according to the enabled/disabled status of the tool.
    if HTConfig:GetIsEnabledBusyResponder(nil) == true then
      brMessageText = string.gsub(brMessageText, "BEGIN_COLOR_TEXT", "|cffffff00")
    else
      brMessageText = string.gsub(brMessageText, "BEGIN_COLOR_TEXT", "|cffffffff")
    end

     -- Set the number color according to the number of messages stored.
    if tonumber(numMessages) > 0 then
      brMessageText = string.gsub(brMessageText, "BEGIN_COLOR_NUM", "|cffffff00")
    else
      brMessageText = string.gsub(brMessageText, "BEGIN_COLOR_NUM", "|cffffffff")
    end

    brMessageText = string.gsub(brMessageText, "NUM_MESSAGES", numMessages)
    brMessageText = string.gsub(brMessageText, "END_COLOR", "|r")

    textLabel = textLabel .. brMessageText

  end

  return textLabel

end

-- ============================================================================
-- == GetTextLabelContentGStatsMembers(textLabel)                            ==
-- ==                                                                        ==
-- == This function adds the number of guild members to the text label.      ==
-- ==                                                                        ==
-- == Example Label Addition: M(12/32) = Members(12 online, 32 total)        ==
-- ============================================================================

function HTBrokerSupport:GetTextLabelContentGStatsMembers(textLabel)

  if HTConfig:GetIsEnabledTextLabelContentGStatsMembers(nil) == true then

    if HTConfig:GetIsEnabledTextLabelContentShowMinimised(nil) == true then
      textLabel = textLabel .. " |cff00ff00" .. HTCommon:GetNumGuildCharactersOnline() .. "|r"
    else
      textLabel = textLabel .. " |cff00ff00" .. L["ldb-textlabel-gs-members"] .. "(" .. HTCommon:GetNumGuildCharactersOnline() .. "/" .. HTCommon:GetNumGuildMembers() .. ")|r"
    end

  end

  return textLabel

end

-- ============================================================================
-- == GetTextLabelContentGStatsRaiders(textLabel)                            ==
-- ==                                                                        ==
-- == This function adds the number of guild raiders to the text label.      ==
-- ==                                                                        ==
-- == Example Label Addition: R(12/32) = Raiders(12 online, 32 total)        ==
-- ============================================================================

function HTBrokerSupport:GetTextLabelContentGStatsRaiders(textLabel)

  if HTConfig:GetIsEnabledTextLabelContentGStatsRaiders(nil) == true then

    if HTConfig:GetIsEnabledTextLabelContentShowMinimised(nil) == true then
      textLabel = textLabel .. " |cffff7d0a" .. HTCommon:GetNumGuildRaidingMembersOnline() .. "|r"
    else
      textLabel = textLabel .. " |cffff7d0a" .. L["ldb-textlabel-gs-raiders"] .. "(" .. HTCommon:GetNumGuildRaidingMembersOnline() .. "/" .. HTCommon:GetNumGuildRaidingMembers() .. ")|r"
    end

  end

  return textLabel

end

-- ============================================================================
-- == GetTextLabelContentGStatsOfficers(textLabel)                           ==
-- ==                                                                        ==
-- == This function adds the number of guild officers to the text label.     ==
-- ==                                                                        ==
-- == Example Label Addition: O(12/32) = Officers(12 online, 32 total)       ==
-- ============================================================================

function HTBrokerSupport:GetTextLabelContentGStatsOfficers(textLabel)

  if HTConfig:GetIsEnabledTextLabelContentGStatsOfficers(nil) == true then

    if HTConfig:GetIsEnabledTextLabelContentShowMinimised(nil) == true then
      textLabel = textLabel .. " |cff00c0ff" .. HTCommon:GetNumGuildOfficersOnline() .. "|r"
    else
      textLabel = textLabel .. " |cff00c0ff" .. L["ldb-textlabel-gs-officers"] .. "(" .. HTCommon:GetNumGuildOfficersOnline() .. "/" .. HTCommon:GetNumGuildOfficers() .. ")|r"
    end

  end

  return textLabel

end

-- ============================================================================
-- == GetTextLabelContentGStatsCharacters(textLabel)                         ==
-- ==                                                                        ==
-- == This function adds the number of guild officers to the text label.     ==
-- ==                                                                        ==
-- == Example Label Addition: C(12/32) = Characters(12 online, 32 total)     ==
-- ============================================================================

function HTBrokerSupport:GetTextLabelContentGStatsCharacters(textLabel)

  if HTConfig:GetIsEnabledTextLabelContentGStatsCharacters(nil) == true then

    if HTConfig:GetIsEnabledTextLabelContentShowMinimised(nil) == true then
      textLabel = textLabel .. " |cffffffff" .. HTCommon:GetNumGuildCharactersOnline() .. "|r"
    else
      textLabel = textLabel .. " |cffffffff" .. L["ldb-textlabel-gs-characters"] .. "(" .. HTCommon:GetNumGuildCharactersOnline() .. "/" .. HTCommon:GetNumGuildCharacters() .. ")|r"
    end

  end

  return textLabel

end

-- ============================================================================
-- == GetTextLabelContentGuildPlayMode(textLabel)                            ==
-- ==                                                                        ==
-- == This function adds the current guild play mode to the text label.      ==
-- ==                                                                        ==
-- == Example Label Addition: P(O|I|R) = Play Mode(Offtime|Invites|Raiding)  ==
-- ============================================================================

function HTBrokerSupport:GetTextLabelContentGuildPlayMode(textLabel)

  if HTConfig:GetIsEnabledTextLabelContentGuildPlayMode(nil) == true then

    local playMode     = HTCommon:GetCurrentGuildPlayMode()
    local playModeName = HTCommon:GetCurrentGuildPlayModeName()

    if playMode == L["cmn-guild-playmode-invites"] then
      -- Text colour is 'orange' ala raid-chat.
      textLabel = textLabel .. " |cffff7d0a" .. playModeName .. "|r"
    elseif playMode == L["cmn-guild-playmode-raiding"] then
      -- Text colour is 'yellow' as an attention prompt.
      textLabel = textLabel .. " |cffff00ff" .. playModeName .. "|r"
    else
      -- Text colour is white, the 'normal' colour.
      textLabel = textLabel .. " |cffffffff" .. playModeName .. "|r"
    end

  end

  return textLabel

end

-- ============================================================================
-- == GetTextLabelContentCombatStatus(textLabel)                             ==
-- ==                                                                        ==
-- == This function adds a combat status indicator to the text label.        ==
-- ==                                                                        ==
-- == Example Label Addition: IC = In combat, OOC = Out of Combat.           ==
-- ============================================================================

function HTBrokerSupport:GetTextLabelContentCombatStatus(textLabel)

  if HTConfig:GetIsEnabledTextLabelContentCombatStatus(nil) == true then

    if HTConfig:GetIsPlayerInCombat(nil) == true then
      textLabel = textLabel .. " |cffff0000" .. L["ldb-textlabel-combatstatus-in"]  .. "|r"
    else
      textLabel = textLabel .. " |cffffffff" .. L["ldb-textlabel-combatstatus-out"] .. "|r"
    end

  end

  return textLabel

end

-- ============================================================================
-- == GetTextLabelContentAddonName(textLabel)                                ==
-- ==                                                                        ==
-- == This function fetches the addon name, the simplest text label content. ==
-- == The addon name is only displayed if no other text label content is     ==
-- == currently selected.                                                    ==
-- ============================================================================

function HTBrokerSupport:GetTextLabelContentAddonName(textLabel)

  if textLabel == "" then

    textLabel = textLabel.. L["addon-name"]

  end

  return textLabel

end

-- ============================================================================
-- == BuildTooltip(tooltip)                                                  ==
-- ==                                                                        ==
-- == This function builds the tooltip object displayed by the data object.  ==
-- ============================================================================

function HTBrokerSupport:BuildTooltip(tooltip)

  -- Return immediately if tooltip is not supplied / not a tooltip object.
  if not tooltip or not tooltip.AddLine then return end

  HTBrokerSupport:AddTooltipAddonName(tooltip)
  HTBrokerSupport:AddTooltipAddonDescription(tooltip)
  HTBrokerSupport:AddTooltipContent(tooltip)
  HTBrokerSupport:AddTooltipContentButtonActions(tooltip)

end

-- ============================================================================
-- == AddTooltipAddonName(tooltip)                                           ==
-- ==                                                                        ==
-- == This function adds the addon-name to the tooltip.                      ==
-- ============================================================================

function HTBrokerSupport:AddTooltipAddonName(tooltip)

  tooltip:AddLine("|r" .. L["addon-name"] .. " (" .. L["addon-version-number"] .. ")|r")
  tooltip:AddLine(" ")

end

-- ============================================================================
-- == AddTooltipAddonDescription(tooltip)                                    ==
-- ==                                                                        ==
-- == This function adds the addon-description to the tooltip.               ==
-- ============================================================================

function HTBrokerSupport:AddTooltipAddonDescription(tooltip)

  tooltip:AddLine(L["addon-desc-01"])
  tooltip:AddLine(L["addon-desc-02"])
  tooltip:AddLine(" ")

end

-- ============================================================================
-- == AddTooltipContent(tooltip)                                             ==
-- ==                                                                        ==
-- == This function adds the main content to the tooltip.                    ==
-- ============================================================================

function HTBrokerSupport:AddTooltipContent(tooltip)

  HTBrokerSupport:AddTooltipContentGuildInformation(tooltip)
  HTBrokerSupport:AddTooltipContentGuildStatistics(tooltip)
  HTBrokerSupport:AddTooltipContentGMOTD(tooltip)
  HTBrokerSupport:AddTooltipContentOfficerList(tooltip)
  HTBrokerSupport:AddTooltipContentToolStatusInfo(tooltip)

  -- If NO content is currently enabled, set the tooltip to show a minimal
  -- "there is no tooltip content" message.
  if  not HTConfig:GetIsEnabledTooltipContentGuildInformation(nil)
  and not HTConfig:GetIsEnabledTooltipContentGuildStatistics(nil)
  and not HTConfig:GetIsEnabledTooltipContentGMOTD(nil)
  and not HTConfig:GetIsEnabledTooltipContentOfficerList(nil)
  and not HTConfig:GetIsEnabledTooltipContentToolStatusInfo(nil) then
    HTBrokerSupport:AddTooltipContentNoContentEnabled(tooltip)
  end

end

-- ============================================================================
-- == AddTooltipContentGuildInformation(tooltip)                             ==
-- ==                                                                        ==
-- == This function adds guild information content to the tooltip.           ==
-- ============================================================================

function HTBrokerSupport:AddTooltipContentGuildInformation(tooltip)

  -- Do not proceed if the configuration for this feature is not enabled.
  if not HTConfig:GetIsEnabledTooltipContentGuildInformation(nil) then return end

  tooltip:AddLine("|cff00c0ff" .. L["ldb-tooltip-heading-guildinfo"] .. "|r")

  tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-name"]  .. "|r", "|cff00ff00" .. HTCommon:GetGuildName()     .."|r")
  tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-rank"]  .. "|r", "|cff00ff00" .. HTCommon:GetGuildRankName() .."|r")
  tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-level"] .. "|r", "|cff00ff00" .. HTCommon:GetGuildLevel()    .."|r")

  tooltip:AddLine(" ")

end

-- ============================================================================
-- == AddTooltipContentGuildStatistics(tooltip)                              ==
-- ==                                                                        ==
-- == This function adds guild statistics content to the tooltip.            ==
-- ============================================================================

function HTBrokerSupport:AddTooltipContentGuildStatistics(tooltip)

  -- Do not proceed if the configuration for this feature is not enabled.
  if not HTConfig:GetIsEnabledTooltipContentGuildStatistics(nil) then return end

  tooltip:AddLine("|cff00c0ff" .. L["ldb-tooltip-heading-guildstats"] .. "|r")

  tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-members"]    .. "|r", "|cff00ff00" .. HTCommon:GetNumGuildCharactersOnline()     .."/" .. HTCommon:GetNumGuildMembers()        .. "|r")
  tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-raiders"]    .. "|r", "|cff00ff00" .. HTCommon:GetNumGuildRaidingMembersOnline() .."/" .. HTCommon:GetNumGuildRaidingMembers() .. "|r")
  tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-officers"]   .. "|r", "|cff00ff00" .. HTCommon:GetNumGuildOfficersOnline()       .."/" .. HTCommon:GetNumGuildOfficers()       .. "|r")
  tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-characters"] .. "|r", "|cff00ff00" .. HTCommon:GetNumGuildCharactersOnline()     .."/" .. HTCommon:GetNumGuildCharacters()     .. "|r")

  tooltip:AddLine(" ")

end

-- ============================================================================
-- == AddTooltipContentGMOTD(tooltip)                                        ==
-- ==                                                                        ==
-- == This function adds the GMOTD content to the tooltip.                   ==
-- ============================================================================

function HTBrokerSupport:AddTooltipContentGMOTD(tooltip)

  -- Do not proceed if the configuration for this feature is not enabled.
  if not HTConfig:GetIsEnabledTooltipContentGMOTD(nil) then return end

  local maxWidth        = HTConfig:GetChatWindowTextMaxWidth(nil)
  local words           = HTData:StringSplit(HTData:SafeString(GetGuildRosterMOTD()), " ")
  local characterCount  = 0
  local thisLineText    = ""
  local spacerChar      = ""

  tooltip:AddLine("|cff00c0ff" .. L["ldb-tooltip-heading-gmotd"] .. "|r")

  -- Loop through the list of words, printing as many per line as the maximum
  -- line width allows.
  for i,v in ipairs(words) do
    if (characterCount + string.len(v)) < maxWidth then
      thisLineText   = thisLineText .. spacerChar .. v
      characterCount = characterCount + string.len(v) + 1
      spacerChar     = " "
    else
      tooltip:AddLine("|cff00ff00" .. thisLineText .. "|r")
      thisLineText      = v .. spacerChar
      characterCount    = 0 + string.len(v) + 2
      spacerChar        = ""
    end
  end

  -- Check if there are any words remaining to be printed as a final non-full
  -- line.
  if string.len(thisLineText) > 0 then
    tooltip:AddLine("|cff00ff00" .. thisLineText .. "|r")
  end

  tooltip:AddLine(" ")

  -- Determine the proper guild play mode value.
  local guildPlayMode = L["ldb-tooltip-item-text-playmode-offtime"]

  if HTCommon:GetCurrentGuildPlayMode() == L["cmn-guild-playmode-invites"] then
    guildPlayMode = L["ldb-tooltip-item-text-playmode-invites"]
  elseif HTCommon:GetCurrentGuildPlayMode() == L["cmn-guild-playmode-raiding"] then
    guildPlayMode = L["ldb-tooltip-item-text-playmode-raiding"]
  end

  tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-playmode"] .. "|r", "|cff00ff00" .. guildPlayMode .."|r")

  tooltip:AddLine(" ")

end

-- ============================================================================
-- == AddTooltipContentOfficerList(tooltip)                                  ==
-- ==                                                                        ==
-- == This function adds officer-list content to the tooltip.                ==
-- ============================================================================

function HTBrokerSupport:AddTooltipContentOfficerList(tooltip)

  -- Do not proceed if the configuration for this feature is not enabled.
  if not HTConfig:GetIsEnabledTooltipContentOfficerList(nil) then return end

  tooltip:AddLine("|cff00c0ff" .. L["ldb-tooltip-heading-guildofficers"] .. "|r")

  local guildOfficerList = HTBrokerSupport:GetTooltipOfficerList()

  for i,v in ipairs(guildOfficerList) do

    local parts       = HTData:StringSplit(v, L["all-text-separator"])
    local officerName = parts[1]
    local timeOffline = parts[2]

    if timeOffline == L["all-data-unset"] then
      timeOffline = L["con-findbycharname-online"]
    end

    tooltip:AddDoubleLine(officerName, timeOffline)

  end

  tooltip:AddLine(" ")

end

-- ============================================================================
-- == AddTooltipContentToolStatusInfo(tooltip)                               ==
-- ==                                                                        ==
-- == This function adds tool status info content to the tooltip.            ==
-- ============================================================================

function HTBrokerSupport:AddTooltipContentToolStatusInfo(tooltip)

  -- Do not proceed if the configuration for this feature is not enabled.
  if not HTConfig:GetIsEnabledTooltipContentToolStatusInfo(nil) then return end

  tooltip:AddLine("|cff00c0ff" .. L["ldb-tooltip-heading-tools"] .. "|r")
  tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-autoinvitations"]  .. "|r", HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoInvitations(nil)))
  tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-buffannouncer"]    .. "|r", HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffAnnouncer(nil)))
  tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-buffmonitor"]      .. "|r", HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitor(nil)))
  tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-busyresponder"]    .. "|r", "|cffffff00" .. " (".. HTConsole:BusyResponderGetNumberOfMessagesStored() .. L["ldb-tooltip-item-text-busyresponder"] .. ") " .. "|r" .. HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBusyResponder(nil)) )
  tooltip:AddDoubleLine("|cffffffff" .. "- " .. L["ldb-tooltip-item-name-br-profile"]  .. "|r", "|cffffff00" .. HTConfig:GetBRProfileName(nil) .. "|r")

  tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-rms-timer"]  .. "|r", "|cffffff00" .. " (".. HTData:SecondsToMinutes(HTConsole:GetRMSTimer()) .. ") " .. "|r" .. HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledRecruitmentMessageSender(nil)))
  tooltip:AddDoubleLine("|cffffffff" .. "- " .. L["ldb-tooltip-item-name-rms-grouplock"]  .. "|r",                 HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledRMSGroupLock(nil)))
  tooltip:AddDoubleLine("|cffffffff" .. "- " .. L["ldb-tooltip-item-name-rms-prefguild"]  .. "|r", "|cffffff00" .. HTConfig:GetPreferredRMSGuild(nil) .. "|r")
  tooltip:AddDoubleLine("|cffffffff" .. "- " .. L["ldb-tooltip-item-name-rms-channel"]    .. "|r", "|cffffff00" .. HTConsole:GetRMSChannelNumber()    .. " (" .. HTConsole:GetRMSChannelNameByNumber(HTConsole:GetRMSChannelNumber()) .. ")|r")

  tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-heartbeat-timer"]  .. "|r", "|cffffff00" .. " (".. HTData:SecondsToMinutes(HTConsole:GetHeartbeatTimer()) .. " / " ..  HTData:SecondsToMinutes(HTConfig:GetHeartbeatInterval(nil)) .. ")|r")

  if HTConfig:GetIsEnabledDebugMode() then
    tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-debugmode-timer"]    .. "|r", HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledDebugMode(nil)))
    tooltip:AddDoubleLine("|cffffffff" .. L["ldb-tooltip-item-name-debugfilter-timer"]  .. "|r", "|cffffff00" .. " (".. HTConfig:GetDebugFilter(nil) .. ") " .. "|r" .. HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledDebugFilter(nil)))
  end

  tooltip:AddLine(" ")

end

-- ============================================================================
-- == AddTooltipContentNoContentEnabled(tooltip)                             ==
-- ==                                                                        ==
-- == This function adds the main content to the tooltip, indicating that    ==
-- == no content has been enabled in the configuration preferences.          ==
-- ============================================================================

function HTBrokerSupport:AddTooltipContentNoContentEnabled(tooltip)

  tooltip:AddLine("|cff00c0ff" .. L["ldb-tooltip-heading-nothing-enabled"] .. "|r")

  tooltip:AddLine("|cffff0000" .. L["ldb-tootip-content-nothing-enabled-01"] .. "|r")
  tooltip:AddLine("|cffff0000" .. L["ldb-tootip-content-nothing-enabled-02"] .. "|r")
  tooltip:AddLine("|cffff0000" .. L["ldb-tootip-content-nothing-enabled-03"] .. "|r")

  tooltip:AddLine(" ")

end

-- ============================================================================
-- == AddTooltipContentButtonActions(tooltip)                                ==
-- ==                                                                        ==
-- == This function adds the button-actions to the tooltip.                  ==
-- ============================================================================

function HTBrokerSupport:AddTooltipContentButtonActions(tooltip)

  -- Do not proceed if the configuration for this feature is not enabled.
  if not HTConfig:GetIsEnabledTooltipContentButtonActions(nil) then return end

  -- Button Actions Heading.
  tooltip:AddLine("|cff00c0ff" .. L["ldb-tooltip-heading-actions"] .. "|r")

  -- Left Click Button Actions.
  HTBrokerSupport:AddTooltipContentButtonAction(L["ldb-tooltip-item-label-left"], false, false, L["ldb-tooltip-item-value-left"],          tooltip)
  HTBrokerSupport:AddTooltipContentButtonAction(L["ldb-tooltip-item-label-left"], true,  false, L["ldb-tooltip-item-value-left-alt"],      tooltip) -- alt
  HTBrokerSupport:AddTooltipContentButtonAction(L["ldb-tooltip-item-label-left"], false, true,  L["ldb-tooltip-item-value-left-ctrl"],     tooltip) -- ctrl
  HTBrokerSupport:AddTooltipContentButtonAction(L["ldb-tooltip-item-label-left"], true,  true,  L["ldb-tooltip-item-value-left-ctrl-alt"], tooltip) -- ctrl+alt

  -- Right Click Button Actions.
  tooltip:AddLine(" ")
  HTBrokerSupport:AddTooltipContentButtonAction(L["ldb-tooltip-item-label-right"], false, false, L["ldb-tooltip-item-value-right"],          tooltip)
  HTBrokerSupport:AddTooltipContentButtonAction(L["ldb-tooltip-item-label-right"], true,  false, L["ldb-tooltip-item-value-right-alt"],      tooltip) -- alt
  HTBrokerSupport:AddTooltipContentButtonAction(L["ldb-tooltip-item-label-right"], false, true,  L["ldb-tooltip-item-value-right-ctrl"],     tooltip) -- ctrl
  HTBrokerSupport:AddTooltipContentButtonAction(L["ldb-tooltip-item-label-right"], true,  true,  L["ldb-tooltip-item-value-right-ctrl-alt"], tooltip) -- ctrl+alt

  tooltip:AddLine(" ")

end

-- ============================================================================
-- == AddTooltipContentButtonAction(buttonLabel, isAltActive, isCtrlActive,  ==
-- ==                               buttonValue, tooltip)                    ==
-- ==                                                                        ==
-- == This function adds a single button-action to the tooltip.              ==
-- ==                                                                        ==
-- == Parameters:                                                            ==
-- ==                                                                        ==
-- == buttonLabel:  The name of the button clicked. (eg: 'Left' or 'Right')  ==
-- == isAltActive:  True if Alt key depressed, false otherwise.              ==
-- == isCtrlActive: True if Ctrl key depressed, false otherwise.             ==
-- == buttonValue:  The action this button-click triggers. (eg: 'Show Menu') ==
-- == tooltip:      The tooltip object to add this button action info to.    ==
-- ============================================================================

function HTBrokerSupport:AddTooltipContentButtonAction(buttonLabel, isAltActive, isCtrlActive, buttonValue, tooltip)

  local activeColour   = "|cffeda55f"
  local inactiveColour = "|cff333333"
  local valueColour    = "|cff00ff00"
  local resetColour    = "|r"

  local labelText = activeColour .. buttonLabel .. resetColour
  local valueText = activeColour .. buttonValue .. resetColour

  if isAltActive then
    labelText = labelText .. activeColour   .. L["ldb-tooltip-item-plus-alt"] .. resetColour
  else
    labelText = labelText .. inactiveColour .. L["ldb-tooltip-item-plus-alt"] .. resetColour
  end

  if isCtrlActive then
    labelText = labelText .. activeColour   .. L["ldb-tooltip-item-plus-ctl"] .. resetColour
  else
    labelText = labelText .. inactiveColour .. L["ldb-tooltip-item-plus-ctl"] .. resetColour
  end

  tooltip:AddDoubleLine(labelText, valueText)

end

-- ============================================================================
-- == ShowMenu()                                                             ==
-- ==                                                                        ==
-- == This function displays the drop-down menu.                             ==
-- ============================================================================

function HTBrokerSupport:ShowMenu()

  HTBrokerSupport:BuildDropDownMenu()

  ToggleDropDownMenu(1, nil, DropDownMenu, "cursor", 0, 0)

end

-- ============================================================================
-- == BuildDropDownMenu()                                                    ==
-- ==                                                                        ==
-- == This function builds the drop-down menu. It is a dynamic function and  ==
-- == can be called either once only for menus that are not expected to      ==
-- == change during a session or every time before the menu is displayed for ==
-- == menus that change often.                                               ==
-- ============================================================================

function HTBrokerSupport:BuildDropDownMenu()

  DropDownMenu.displayMode = "MENU"
  DropDownMenu.initialize  = function(self, level)

    if not level then return end
    wipe(info)

    -- ========================================================================
    -- == Level 1 - Top level menu items.                                    ==
    -- ========================================================================
    if level == 1 then

      -- General Menu Defaults.
      HTBrokerSupport:AddMenuDefaultConfig()

      -- Create the title of the menu
      HTBrokerSupport:AddMenuTitle(L["ldb-menu-main-name"])
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Buff-Announcer Submenu.
      HTBrokerSupport:AddMenuSubMenu("SubmenuBuffAnnouncer")
      HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-buffannouncer")
      UIDropDownMenu_AddButton(info, level)

      -- Buff-Monitor Submenu.
      HTBrokerSupport:AddMenuSubMenu("SubmenuBuffMonitor")
      HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-buffmonitor")
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Busy Responder Submenu.
      HTBrokerSupport:AddMenuSubMenu("SubmenuBusyResponder")
      HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-busyresponder")
      UIDropDownMenu_AddButton(info, level)

      -- Guild Management Submenu.
      HTBrokerSupport:AddMenuSubMenu("SubmenuGuildManagement")
      HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-guildmanagement")
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Invitations Submenu.
      HTBrokerSupport:AddMenuSubMenu("SubmenuInvitations")
      HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-invitations")
      UIDropDownMenu_AddButton(info, level)

      -- Raid Functions Submenu.
      HTBrokerSupport:AddMenuSubMenu("SubmenuRaidFunctions")
      HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-raid")
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- Recruitment Submenu.
      HTBrokerSupport:AddMenuSubMenu("SubmenuRecruitment")
      HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-recruitment")
      UIDropDownMenu_AddButton(info, level)

      -- System Submenu.
      HTBrokerSupport:AddMenuSubMenu("SubmenuSystem")
      HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-system")
      UIDropDownMenu_AddButton(info, level)

      -- Empty Menu Item.
      HTBrokerSupport:AddMenuDummyButton()
      UIDropDownMenu_AddButton(info, level)

      -- About Menu Item.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-about")
      info.func = function() HTGUI:About() end
      UIDropDownMenu_AddButton(info, level)

      -- Close Menu Item.
      HTBrokerSupport:AddMenuPushButton()
      HTBrokerSupport:AddMenuButtonText("ldb-menu-item-closemenu")
      info.func = function() CloseDropDownMenus() end
      UIDropDownMenu_AddButton(info, level)

    -- ========================================================================
    -- == Level 2 - Submenus directly linked from the main menu.             ==
    -- ========================================================================
    elseif level == 2 then

      -- ======================================================================
      -- == Render the level 2 submenu for: Buff-Announcer ====================
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuBuffAnnouncer" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-buffannouncer-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-quick-config-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Do BuffAnnouncer Enabled Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-ba-enabled")
        info.checked = function() return HTConfig:GetIsEnabledBuffAnnouncer(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBuffAnnouncer()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-ba-enabled-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffAnnouncer(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Do BuffAnnouncer Food Buff Deployed Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-ba-food-enabled")
        info.checked = function() return HTConfig:GetIsEnabledBuffAnnouncerFood(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBuffAnnouncerFood()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-ba-food-enabled-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffAnnouncerFood(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Do BuffAnnouncer Cauldron Deployed Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-ba-cauldron-enabled")
        info.checked = function() return HTConfig:GetIsEnabledBuffAnnouncerCauldron(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBuffAnnouncerCauldron()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-ba-cauldron-enabled-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffAnnouncerCauldron(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Do BuffAnnouncer Soulwell (Healthstones) Deployed Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-ba-healthstone-enabled")
        info.checked = function() return HTConfig:GetIsEnabledBuffAnnouncerHealthStones(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBuffAnnouncerHealthstones()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-ba-healthstone-enabled-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffAnnouncerHealthstones(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Do BuffAnnouncer Food Table (Ritual of Refreshment) Deployed Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-ba-foodtable-enabled")
        info.checked = function() return HTConfig:GetIsEnabledBuffAnnouncerFoodTable(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBuffAnnouncerFoodTable()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-ba-foodtable-enabled-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffAnnouncerFoodTable(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Do BuffAnnouncer Repair Bot Deployed Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-ba-repairbot-enabled")
        info.checked = function() return HTConfig:GetIsEnabledBuffAnnouncerRepairBot(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBuffAnnouncerRepairBot()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-ba-repairbot-enabled-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffAnnouncerRepairBot(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

      end

      -- ======================================================================
      -- == Render the level 2 submenu for: Buff-Monitor ======================
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuBuffMonitor" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-buffmonitor-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-manual-checks-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Do Self Check.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-doselfbuffcheck")
        info.func = function() HTBuffMonitor:DoManualSelfCheck() end
        UIDropDownMenu_AddButton(info, level)

        -- Do Raid Check to RAID channel.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-doraidbuffcheck-raid")
        info.func = function() HTBuffMonitor:DoRaidCheck("RAID") end
        UIDropDownMenu_AddButton(info, level)

        -- Do Raid Check to CHAT channel.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-doraidbuffcheck-chat")
        info.func = function() HTBuffMonitor:DoRaidCheck("CHAT") end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-automatic-checks-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-quick-config-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Do BuffMonitor Enabled Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-bm-enabled")
        info.checked = function() return HTConfig:GetIsEnabledBuffMonitor(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBuffMonitor()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-bm-enabled-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitor(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Do BuffMonitor Check-on-Heartbeat Enabled Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-bm-on-heartbeat")
        info.checked = function() return HTConfig:GetIsEnabledBuffMonitorCheckOnHeartbeat(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBuffMonitorCheckOnHeartbeat()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-bm-on-heartbeat-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCheckOnHeartbeat(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Do BuffMonitor Check-on-Ready-Check Enabled Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-bm-on-readycheck")
        info.checked = function() return HTConfig:GetIsEnabledBuffMonitorCheckOnReadyCheck(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBuffMonitorCheckOnReadyCheck()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-bm-on-readycheck-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCheckOnReadyCheck(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Do BuffMonitor Check-on-Combat-Start Enabled Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-bm-on-combatcheck")
        info.checked = function() return HTConfig:GetIsEnabledBuffMonitorCheckOnCombatStart(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBuffMonitorCheckOnCombatStart()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-bm-on-combatcheck-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCheckOnCombatStart(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Do Mists Buffs Acceptable Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-bm-inc-mists")
        info.checked = function() return HTConfig:GetIsEnabledBuffMonitorMistsBuffs(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBuffMonitorMistsBuffs()
                         HTConfig:UpdateBuffMonitorByExpansion()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-bm-inc-mists-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorMistsBuffs(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Do Cataclysm Buffs Acceptable Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-bm-inc-cata")
        info.checked = function() return HTConfig:GetIsEnabledBuffMonitorCataBuffs(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBuffMonitorCataBuffs()
                         HTConfig:UpdateBuffMonitorByExpansion()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-bm-inc-cata-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCataBuffs(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Do Wrath Buffs Acceptable Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-bm-inc-wrath")
        info.checked = function() return HTConfig:GetIsEnabledBuffMonitorWrathBuffs(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBuffMonitorWrathBuffs()
                         HTConfig:UpdateBuffMonitorByExpansion()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-bm-inc-wrath-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorWrathBuffs(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Do Flask Check Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-doflaskcheck")
        info.checked = function() return HTConfig:GetIsEnabledBuffMonitorCheckFlasks(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBuffMonitorCheckFlasks()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-doflaskcheck-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCheckFlasks(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Do Battle Elixir Check Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dobelixircheck")
        info.checked = function() return HTConfig:GetIsEnabledBuffMonitorCheckBattleElixirs(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBuffMonitorCheckBattleElixirs()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-dobelixircheck-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCheckBattleElixirs(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

         -- Do Guardian Elixir Check Toggle.
         HTBrokerSupport:AddMenuToggleButton()
         HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dogelixircheck")
         info.checked = function() return HTConfig:GetIsEnabledBuffMonitorCheckGuardianElixirs(nil) end
         info.func    = function()
                          HTConfig:ToggleIsEnabledBuffMonitorCheckGuardianElixirs()
                          HTBrokerSupport:DisplayStatus(L["ldb-menu-item-dogelixircheck-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCheckGuardianElixirs(nil)))
                        end
        UIDropDownMenu_AddButton(info, level)

         -- Do Rogue Poisons Check Toggle.
         HTBrokerSupport:AddMenuToggleButton()
         HTBrokerSupport:AddMenuButtonText("ldb-menu-item-roguepoisons")
         info.checked = function() return HTConfig:GetIsEnabledBuffMonitorCheckPoisons(nil) end
         info.func    = function()
                          HTConfig:ToggleIsEnabledBuffMonitorCheckPoisons()
                          HTBrokerSupport:DisplayStatus(L["ldb-menu-item-roguepoisons-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBuffMonitorCheckPoisons(nil)))
                        end
        UIDropDownMenu_AddButton(info, level)

      end

      -- ======================================================================
      -- == Render the level 2 submenu for: Busy Responder ====================
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuBusyResponder" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-busyresponder-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Show saved messages.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-showsavedmsgs")
        info.func = function() HTConsole:BusyResponderShowMessages() end
        UIDropDownMenu_AddButton(info, level)

        -- Clear saved messages.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-clearsavedmsgs")
        info.func = function() HTConsole:BusyResponderClearMessages() end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Select Busy Profile.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-profile-select-busy")
        info.func = function() HTConsole:BusyResponderSelectProfileBusy() CloseDropDownMenus() end
        UIDropDownMenu_AddButton(info, level)

        -- Select Raiding Profile.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-profile-select-raiding")
        info.func = function() HTConsole:BusyResponderSelectProfileRaiding() CloseDropDownMenus() end
        UIDropDownMenu_AddButton(info, level)

        -- Select Away Profile.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-profile-select-away")
        info.func = function() HTConsole:BusyResponderSelectProfileAway() CloseDropDownMenus() end
        UIDropDownMenu_AddButton(info, level)

        -- Select Recruiting Profile.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-profile-select-recruiting")
        info.func = function() HTConsole:BusyResponderSelectProfileRecruiting() CloseDropDownMenus() end
        UIDropDownMenu_AddButton(info, level)        
        
        -- Select Custom Profile.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-profile-select-custom")
        info.func = function() HTConsole:BusyResponderSelectProfileCustom() CloseDropDownMenus() end
        UIDropDownMenu_AddButton(info, level)
        
        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Save Current Profile.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-profile-save")
        info.func = function() HTConsole:BusyResponderSaveProfile() end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-quick-config-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Enable Busy Responder.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-brenabled")
        info.checked = function() return HTConfig:GetIsEnabledBusyResponder(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBusyResponder()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-brenabled-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBusyResponder(nil)))
                         -- Update the text-label that other addons may display.
                         HTBrokerSupport:BuildTextLabel()
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Is Guild Member Allowed.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-allowguildmember")
        info.checked = function() return HTConfig:GetIsGuildMemberAllowed(nil) end
        info.func    = function()
                         HTConfig:ToggleIsGuildMemberAllowed()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-allowguildmember-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsGuildMemberAllowed(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

         -- Is Guild Officer Allowed.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-allowguildofficer")
        info.checked = function() return HTConfig:GetIsGuildOfficerAllowed(nil) end
        info.func    = function()
                         HTConfig:ToggleIsGuildOfficerAllowed()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-allowguildofficer-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsGuildOfficerAllowed(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Is Raid Member Allowed.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-allowraidmember")
        info.checked = function() return HTConfig:GetIsRaidMemberAllowed(nil) end
        info.func    = function()
                         HTConfig:ToggleIsRaidMemberAllowed()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-allowraidmember-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsRaidMemberAllowed(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Is Party Member Allowed.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-allowpartymember")
        info.checked = function() return HTConfig:GetIsPartyMemberAllowed(nil) end
        info.func    = function()
                         HTConfig:ToggleIsPartyMemberAllowed()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-allowpartymember-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsPartyMemberAllowed(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Is Friend Allowed.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-allowfriend")
        info.checked = function() return HTConfig:GetIsFriendAllowed(nil) end
        info.func    = function()
                         HTConfig:ToggleIsFriendAllowed()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-allowfriend-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsFriendAllowed(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Is Real-ID Friend Allowed.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-allowrealidfriend")
        info.checked = function() return HTConfig:GetIsRealIDFriendAllowed(nil) end
        info.func    = function()
                         HTConfig:ToggleIsRealIDFriendAllowed()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-allowrealidfriend-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsRealIDFriendAllowed(nil)))
                        end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Is Auto Profile Selection Enabled Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-auto-select-profile")
        info.checked = function() return HTConfig:GetIsEnabledBRAutoProfileSelect(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledBRAutoProfileSelect()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-auto-select-profile-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledBRAutoProfileSelect(nil)))
                        end
        UIDropDownMenu_AddButton(info, level)

      end

      -- ======================================================================
      -- == Render the level 2 submenu for: Raid Functions ====================
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuRaidFunctions" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-raid-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Begin Invitation Period Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuRaidInvitationAnnouncers")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-ria")
        UIDropDownMenu_AddButton(info, level)

        -- Begin Raid Session Period Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuRaidQueueAnnouncers")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-rqa")
        UIDropDownMenu_AddButton(info, level)

        -- End Raid Session Item.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-end-raid")
        info.func = function() HalonaTools:EndRaidSession(UnitName("player")) end
        UIDropDownMenu_AddButton(info, level)

        -- End Raid Session Quietly Item.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-end-raid-quietly")
        info.func = function() HalonaTools:EndRaidSessionQuietly(UnitName("player")) end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Disband Raid Item.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-disband-raid")
        info.func = function() HTCommon:DisbandRaid() end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Begin Show VOIP Account Details Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuShowVoipAccountDetails")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-voip")
        UIDropDownMenu_AddButton(info, level)

        -- Begin Show Kill Order Details Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuShowKillOrderDetails")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-killorder")
        UIDropDownMenu_AddButton(info, level)

        -- Pull Timers Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuPullTimers")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-pull-timers")
        UIDropDownMenu_AddButton(info, level)

        -- Bio-Break Timers Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuBioBreakTimers")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-bio-break-timers")
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Do Ready Check Item.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-do-ready-check")
        info.func = function() HTCommon:DoReadyCheck() end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Auto Convert Invites to Session Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-auto-conv-invites-to-session")
        info.checked = function() return HTConfig:GetIsEnabledAutoConvertRaidInvitesToSession(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledAutoConvertRaidInvitesToSession()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-auto-conv-invites-to-session-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoConvertRaidInvitesToSession(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Auto Set Master Loot Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-auto-set-ml")
        info.checked = function() return HTConfig:GetIsEnabledAutoSetMasterLoot(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledAutoSetMasterLoot()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-auto-set-ml-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoSetMasterLoot(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Auto Select Master Looter Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-auto-select-ml")
        info.checked = function() return HTConfig:GetIsEnabledAutoSelectMasterLooter(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledAutoSelectMasterLooter()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-auto-select-ml-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoSelectMasterLooter(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- ASML Criteria Selection Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuASMLCriteriaSelection")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-asml-selection-criteria")
        UIDropDownMenu_AddButton(info, level)

      end

      -- ======================================================================
      -- == Render the level 2 submenu for: Guild Management ==================
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuGuildManagement" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-guildmanagement-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-gm-character-lists-name"])
        UIDropDownMenu_AddButton(info, level)

        -- List All Officers Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-gm-list-officers")
        info.func = function() HalonaTools:ListAllOfficers() end
        UIDropDownMenu_AddButton(info, level)

        -- List All Members (Active & Retired) Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-gm-list-members-all")
        info.func = function() HalonaTools:ListAllMembers() end
        UIDropDownMenu_AddButton(info, level)

        -- List All Members (Active) Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-gm-list-members-active")
        info.func = function() HalonaTools:ListAllActiveMembers() end
        UIDropDownMenu_AddButton(info, level)

        -- List All Members (Social) Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-gm-list-members-social")
        info.func = function() HalonaTools:ListAllSocialMembers() end
        UIDropDownMenu_AddButton(info, level)

        -- List All Members (Retired) Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-gm-list-members-retired")
        info.func = function() HalonaTools:ListAllRetiredMembers() end
        UIDropDownMenu_AddButton(info, level)

        -- List All Trials Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-gm-list-trials")
        info.func = function() HalonaTools:ListAllTrials() end
        UIDropDownMenu_AddButton(info, level)

        -- List All Trial Durations Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-gm-list-trial-durations")
        info.func = function() HalonaTools:ListAllTrialDurations() end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- List Associates Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuListAssociates")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-list-associates")
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Guild Invitations Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuGuildInvitations")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-guild-invitations")
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-gm-admin-tools-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Run All Guild Admin Officer Checks Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-gm-check-runall")
        info.func = function() HalonaTools:GuildAdminOfficerChecks(false) end
        UIDropDownMenu_AddButton(info, level)

        -- Check Trial Officer Notes Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-gm-check-trioffnotes")
        info.func = function() HalonaTools:GuildAdminCheckTrialMemberOfficerNotes(false) end
        UIDropDownMenu_AddButton(info, level)

        -- Check Trial Durations Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-gm-check-trialdurations")
        info.func = function() HalonaTools:GuildAdminCheckTrialMemberDurations(false) end
        UIDropDownMenu_AddButton(info, level)

        -- Check Alt Public Notes Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-gm-check-altpubnotes")
        info.func = function() HalonaTools:GuildAdminCheckAltCharacterPublicNotes(false) end
        UIDropDownMenu_AddButton(info, level)

        -- Check Member Officer Notes Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-gm-check-memoffnotes")
        info.func = function() HalonaTools:GuildAdminCheckMemberOfficerNotes(false) end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-gm-misc-tools-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Show Rank Tags Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-gm-mt-show-rank-tags")
        info.func = function() HTConfig:ShowRankTags() end
        UIDropDownMenu_AddButton(info, level)

        -- Refresh Halona Rank Tags Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-gm-mt-refresh-rank-tags")
        info.func = function() HTConfig:RefreshHalonaRankTags() end
        UIDropDownMenu_AddButton(info, level)

      end

      -- ======================================================================
      -- == Render the level 2 submenu for: Invitations =======================
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuInvitations" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-invitations-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-quick-config-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Auto Invitations Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-autoinvitations")
        info.checked = function() return HTConfig:GetIsEnabledAutoInvitations(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledAutoInvitations()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-autoinvitations-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoInvitations(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Auto Invitations Guild Only Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-autoinvitations-guildonly")
        info.checked = function() return HTConfig:GetIsEnabledAutoInvitationsGuildOnly(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledAutoInvitationsGuildOnly()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-autoinvitations-guildonly-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoInvitationsGuildOnly(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Auto Raid Promotions Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-autoraidpromotions")
        info.checked = function() return HTConfig:GetIsEnabledAutoRaidPromotions(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledAutoRaidPromotions()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-autoraidpromotions-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoRaidPromotions(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Auto Raid Invitation Announcement Response.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-autoriaresponse")
        info.checked = function() return HTConfig:GetIsEnabledAutoRIAResponse(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledAutoRIAResponse()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-autoriaresponse-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoRIAResponse(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Auto Raid Invitation Announcement Response.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-autorqaresponse")
        info.checked = function() return HTConfig:GetIsEnabledAutoRQAResponse(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledAutoRQAResponse()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-autorqaresponse-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAutoRQAResponse(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)
      end

      -- ======================================================================
      -- == Render the level 2 submenu for: Recruitment =======================
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuRecruitment" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-recruitment-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Set RMS Interval to 05 mins.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-rms-interval-05m")
        info.func = function()
                      HTConfig:SetRecruitmentInterval(300, 300)
                      HTBrokerSupport:DisplayValue(L["ldb-menu-item-rms-interval-05m-txt-name"], 360)
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Set RMS Interval to 10 mins.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-rms-interval-10m")
        info.func = function()
                      HTConfig:SetRecruitmentInterval(600, 600)
                      HTBrokerSupport:DisplayValue(L["ldb-menu-item-rms-interval-10m-txt-name"], 720)
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Set RMS Interval to 15 mins.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-rms-interval-15m")
        info.func = function()
                      HTConfig:SetRecruitmentInterval(900, 900)
                      HTBrokerSupport:DisplayValue(L["ldb-menu-item-rms-interval-15m-txt-name"], 900)
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Send RMS Now Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-manualrms")
        info.func = function()
                      HTConsole:BroadcastMessageRMS()
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Force Send RMS Now Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-forcemanualrms")
        info.func = function()
                      HTConsole:ForceBroadcastMessageRMS()
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-quick-config-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Recruitment Message Sender Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-autorms")
        info.checked = function() return HTConfig:GetIsEnabledRecruitmentMessageSender(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledRecruitmentMessageSender()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-autorms-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledRecruitmentMessageSender(nil)))
                         HTConsole:ToggleRMSTimerHelper()
                       end
        UIDropDownMenu_AddButton(info, level)

        -- RMS Broadcast On Zone Changed Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-rms-zonechanged")
        info.checked = function() return HTConfig:GetIsEnabledRMSBroadcastOnZoneChanged(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledRMSBroadcastOnZoneChanged()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-rms-zonechanged-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledRMSBroadcastOnZoneChanged(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- RMS Group Lock.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-rms-grouplock")
        info.checked = function() return HTConfig:GetIsEnabledRMSGroupLock(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledRMSGroupLock()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-rms-grouplock-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledRMSGroupLock(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- RMS Guild Lock.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-rms-guildlock")
        info.checked = function() return HTConfig:GetIsEnabledRMSGuildLocked(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledRMSGuildLocked()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-rms-guildlock-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledRMSGuildLocked(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Show Preferred Guild Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-rms-showguild")
        info.func = function()
                      HTBrokerSupport:DisplayValue(L["ldb-menu-item-rms-showguild-txt-name"], HTConfig:GetPreferredRMSGuild(nil))
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Set Preferred Guild Button.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-rms-setguild")
        info.func = function()
                      HTConsole:UpdatePreferredRMSGuild()
                      HTBrokerSupport:DisplayValue(L["ldb-menu-item-rms-setguild-txt-name"], HTConfig:GetPreferredRMSGuild(nil))
                    end
        UIDropDownMenu_AddButton(info, level)
      end

      -- ======================================================================
      -- == Render the level 2 submenu for: System ============================
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuSystem" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-system-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Show Report Addon Usage (CHAT).
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-addon-usage-chat")
        info.func = function()
                      HalonaTools:ShowReportUsageStatus("CHAT")
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Show Report Addon Usage (GUILD).
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-addon-usage-guild")
        info.func = function()
                      HalonaTools:ShowReportUsageStatus("GUILD")
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Show Report Addon Usage (RAID).
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-addon-usage-raid")
        info.func = function()
                      HalonaTools:ShowReportUsageStatus("RAID")
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Addon Version Broadcast.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-avb")
        info.func = function()
                      HTConsole:BroadcastMessageAVB()
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Addon Version Query.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-avq")
        info.func = function()
                      HTConsole:BroadcastMessageAVQ(L["ldb-menu-item-avq-messagetext"])
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Set Heartbeat Interval to Once Every 01m.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-heartbeat-01m")
        info.func = function()
                      HTConfig:SetHeartbeatInterval(60, 60)
                      HTBrokerSupport:DisplayValue(L["ldb-menu-item-heartbeat-01m-txt-name"], 60)
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Set Heartbeat Interval to Once Every 05m.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-heartbeat-05m")
        info.func = function()
                      HTConfig:SetHeartbeatInterval(300, 300)
                      HTBrokerSupport:DisplayValue(L["ldb-menu-item-heartbeat-05m-txt-name"], 300)
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Set Heartbeat Interval to Once Every 10m.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-heartbeat-10m")
        info.func = function()
                      HTConfig:SetHeartbeatInterval(600, 600)
                      HTBrokerSupport:DisplayValue(L["ldb-menu-item-heartbeat-10m-txt-name"], 600)
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Set Heartbeat Interval to Once Every 15m.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-heartbeat-15m")
        info.func = function()
                      HTConfig:SetHeartbeatInterval(900, 900)
                      HTBrokerSupport:DisplayValue(L["ldb-menu-item-heartbeat-15m-txt-name"], 900)
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-quick-config-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Alert New Addon Version Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-alertnewaddonversion")
        info.checked = function() return HTConfig:GetIsEnabledAlertNewAddonVersion(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledAlertNewAddonVersion()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-alertnewaddonversion-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledAlertNewAddonVersion(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Debug Mode Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-debugmode")
        info.checked = function() return HTConfig:GetIsEnabledDebugMode(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledDebugMode()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-debugmode-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledDebugMode(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Debug Filter Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-debugfilter")
        info.checked = function() return HTConfig:GetIsEnabledDebugFilter(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledDebugFilter()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-debugfilter-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledDebugFilter(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Debug Filters Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuDebugFilters")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-debugfilters")
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Text Label Options Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuTextLabelOptions")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-textlabel-options")
        UIDropDownMenu_AddButton(info, level)

        -- Tooltip Options Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuTooltipOptions")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-tooltip-options")
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Config Menu Item.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-config")
        info.func = function()
                      HTGUI:Config()
                    end
        UIDropDownMenu_AddButton(info, level)
      end

    -- ========================================================================
    -- == Level 3 - Submenus linked from 2nd level menus.                    ==
    -- ========================================================================
    elseif level == 3 then

      -- ======================================================================
      -- == Render the level 3 submenu for: List Associates ===================
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuListAssociates" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-list-associates-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- List Associates - Guild Death Knights Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuListAssociatesGuildDeathKnights")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-la-death-knights")
        UIDropDownMenu_AddButton(info, level)

        -- List Associates - Guild Druids Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuListAssociatesGuildDruids")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-la-druids")
        UIDropDownMenu_AddButton(info, level)

        -- List Associates - Guild Hunters Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuListAssociatesGuildHunters")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-la-hunters")
        UIDropDownMenu_AddButton(info, level)

        -- List Associates - Guild Mages Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuListAssociatesGuildMages")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-la-mages")
        UIDropDownMenu_AddButton(info, level)

        -- List Associates - Guild Monks Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuListAssociatesGuildMonks")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-la-monks")
        UIDropDownMenu_AddButton(info, level)

        -- List Associates - Guild Paladins Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuListAssociatesGuildPaladins")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-la-paladins")
        UIDropDownMenu_AddButton(info, level)

        -- List Associates - Guild Priests Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuListAssociatesGuildPriests")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-la-priests")
        UIDropDownMenu_AddButton(info, level)

        -- List Associates - Guild Rogues Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuListAssociatesGuildRogues")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-la-rogues")
        UIDropDownMenu_AddButton(info, level)

        -- List Associates - Guild Shaman Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuListAssociatesGuildShaman")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-la-shaman")
        UIDropDownMenu_AddButton(info, level)

        -- List Associates - Guild Warlocks Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuListAssociatesGuildWarlocks")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-la-warlocks")
        UIDropDownMenu_AddButton(info, level)

        -- List Associates - Guild Warriors Submenu.
        HTBrokerSupport:AddMenuSubMenu("SubmenuListAssociatesGuildWarriors")
        HTBrokerSupport:AddMenuButtonText("ldb-menu-heading-la-warriors")
        UIDropDownMenu_AddButton(info, level)

      end

      -- ======================================================================
      -- == Render the level 3 submenu for: Guild Invitations =================
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuGuildInvitations" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-guild-invitations-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Add buttons for all characters of this class.
        local characterList = HTConsole:GetWhisperers()

        table.sort(characterList)

        for i,v in ipairs(characterList) do
          -- Character-Specific Selection Button
          HTBrokerSupport:AddMenuPushButton()
          HTBrokerSupport:AddMenuButtonExplicitText(v, v, L["ldb-menu-item-guild-invitations-select-tip-desc"])
          info.func = function() HTConsole:GuildInvite(v) end
          UIDropDownMenu_AddButton(info, level)
        end

      end

      -- ======================================================================
      -- == Render the level 3 submenu for: Raid Invitation Announcers ========
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuRaidInvitationAnnouncers" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-ria-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-ria-player-name"])
        UIDropDownMenu_AddButton(info, level)

        local playerName = HTCommon:ColourByClass(UnitName("player")) .. UnitName("player") .. "|r"

        -- Begin Invitation Period Item.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonExplicitText(playerName, playerName, L["ldb-menu-item-ria-select-tip-desc"])
        info.func = function() HalonaTools:BeginRaidInvitationPeriod(UnitName("player")) end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-ria-guildmembers-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Add buttons for guild member who is currently online.
        local onlineMembers = {}

        for index = 1, (GetNumGuildMembers(true)) do
          local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);
          if charOnline then
            table.insert(onlineMembers, HTData:RemoveRealm(charName))
          end
        end

        table.sort(onlineMembers)

        for i,v in ipairs(onlineMembers) do
          -- Announcer Item.
          HTBrokerSupport:AddMenuPushButton()
          HTBrokerSupport:AddMenuButtonExplicitText(v, v, L["ldb-menu-item-ria-select-tip-desc"])
          info.func = function() HalonaTools:BeginRaidInvitationPeriod(v) end
          UIDropDownMenu_AddButton(info, level)
        end

      end

      -- ======================================================================
      -- == Render the level 3 submenu for: Raid Queue Announcers =============
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuRaidQueueAnnouncers" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-rqa-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-rqa-player-name"])
        UIDropDownMenu_AddButton(info, level)

        local playerName = HTCommon:ColourByClass(UnitName("player")) .. UnitName("player") .. "|r"
        -- Begin Invitation Period Item.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonExplicitText(playerName, playerName, L["ldb-menu-item-rqa-select-tip-desc"])
        info.func = function() HalonaTools:BeginRaidSession(UnitName("player")) end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-rqa-guildmembers-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Add buttons for guild member who is currently online.
        local onlineMembers = {}

        for index = 1, (GetNumGuildMembers(true)) do
          local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);
          if charOnline then
            table.insert(onlineMembers, HTData:RemoveRealm(charName))
          end
        end

        table.sort(onlineMembers)

        for i,v in ipairs(onlineMembers) do
          -- Announcer Item.
          HTBrokerSupport:AddMenuPushButton()
          HTBrokerSupport:AddMenuButtonExplicitText(v, v, L["ldb-menu-item-rqa-select-tip-desc"])
          info.func = function() HalonaTools:BeginRaidSession(v) end
          UIDropDownMenu_AddButton(info, level)
        end

      end

      -- ======================================================================
      -- == Render the level 3 submenu for: Show VOIP Account Details =========
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuShowVoipAccountDetails" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-voip-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Show in CHAT channel.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-voip-chat")
        info.func = function()
                      HalonaTools:ShowVoipAccountDetails("CHAT")
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Show in GUILD channel.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-voip-guild")
        info.func = function()
                      HalonaTools:ShowVoipAccountDetails("GUILD")
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Show in PARTY channel.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-voip-party")
        info.func = function()
                      HalonaTools:ShowVoipAccountDetails("PARTY")
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Show in RAID channel.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-voip-raid")
        info.func = function()
                      HalonaTools:ShowVoipAccountDetails("RAID")
                    end
        UIDropDownMenu_AddButton(info, level)

      end

      -- ======================================================================
      -- == Render the level 3 submenu for: Show Kill Order Details ===========
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuShowKillOrderDetails" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-killorder-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Show in CHAT channel.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-killorder-chat")
        info.func = function()
                      HalonaTools:ShowKillOrderDetails("CHAT")
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Show in GUILD channel.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-killorder-guild")
        info.func = function()
                      HalonaTools:ShowKillOrderDetails("GUILD")
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Show in PARTY channel.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-killorder-party")
        info.func = function()
                      HalonaTools:ShowKillOrderDetails("PARTY")
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Show in RAID channel.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-killorder-raid")
        info.func = function()
                      HalonaTools:ShowKillOrderDetails("RAID")
                    end
        UIDropDownMenu_AddButton(info, level)

      end

      -- ======================================================================
      -- == Render the level 3 submenu for: Pull Timers =======================
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuPullTimers" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-pull-timers-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- DBM Timer, Raid Break, 3 mins.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-pull-10s")
        info.func = function()
                      HalonaTools:CreateDBMPullTimer(10, L["ht-raid-pull-timer-01"], true)
                      HTConsole:SendChatMessage(string.gsub(L["ht-raid-pull-timer-02"], "TIMER_DURATION", "10s"), "RAID", nil, nil)
                    end
        UIDropDownMenu_AddButton(info, level)

      end

      -- ======================================================================
      -- == Render the level 3 submenu for: Bio-Break Timers ==================
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuBioBreakTimers" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-bio-break-timers-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- DBM Timer, Raid Break, 3 mins.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-03m")
        info.func = function()
                      HalonaTools:CreateDBMTimer(3, L["ht-raid-break-timer-01"], true)
                      HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "3"), "RAID", nil, nil)
                    end
        UIDropDownMenu_AddButton(info, level)

        -- DBM Timer, Raid Break, 5 mins.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-05m")
        info.func = function()
                      HalonaTools:CreateDBMTimer(5, L["ht-raid-break-timer-01"], true)
                      HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "5"), "RAID", nil, nil)
                    end
        UIDropDownMenu_AddButton(info, level)

        -- DBM Timer, Raid Break, 10 mins.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-10m")
        info.func = function()
                      HalonaTools:CreateDBMTimer(10, L["ht-raid-break-timer-01"], true)
                      HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "10"), "RAID", nil, nil)
                    end
        UIDropDownMenu_AddButton(info, level)

        -- DBM Timer, Raid Break, 15 mins.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-15m")
        info.func = function()
                      HalonaTools:CreateDBMTimer(15, L["ht-raid-break-timer-01"], true)
                      HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "15"), "RAID", nil, nil)
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- DBM Timer, Raid Break, 20 mins.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-20m")
        info.func = function()
                      HalonaTools:CreateDBMTimer(20, L["ht-raid-break-timer-01"], true)
                      HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "20"), "RAID", nil, nil)
                    end
        UIDropDownMenu_AddButton(info, level)

        -- DBM Timer, Raid Break, 25 mins.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-25m")
        info.func = function()
                      HalonaTools:CreateDBMTimer(25, L["ht-raid-break-timer-01"], true)
                      HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "25"), "RAID", nil, nil)
                    end
        UIDropDownMenu_AddButton(info, level)

        -- DBM Timer, Raid Break, 30 mins.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-dbm-timer-30m")
        info.func = function()
                      HalonaTools:CreateDBMTimer(30, L["ht-raid-break-timer-01"], true)
                      HTConsole:SendChatMessage(string.gsub(L["ht-raid-break-timer-02"], "TIMER_DURATION", "30"), "RAID", nil, nil)
                    end
        UIDropDownMenu_AddButton(info, level)

      end

      -- ======================================================================
      -- == Render the level 3 submenu for: ASML Selection Criteria ===========
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuASMLCriteriaSelection" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-asml-selection-criteria-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- ASML Raid Leader Test Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-asml-test-rl")
        info.checked = function() return HTConfig:GetIsEnabledASMLRaidLeaderTest(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledASMLRaidLeaderTest()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-asml-test-rl-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledASMLRaidLeaderTest(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- ASML Raid Assistant Test Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-asml-test-ra")
        info.checked = function() return HTConfig:GetIsEnabledASMLRaidAssistantTest(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledASMLRaidAssistantTest()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-asml-test-ra-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledASMLRaidAssistantTest(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- ASML Guild Master Test Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-asml-test-gm")
        info.checked = function() return HTConfig:GetIsEnabledASMLGuildMasterTest(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledASMLGuildMasterTest()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-asml-test-gm-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledASMLGuildMasterTest(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- ASML Guild Officer Test Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-asml-test-go")
        info.checked = function() return HTConfig:GetIsEnabledASMLGuildOfficerTest(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledASMLGuildOfficerTest()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-asml-test-go-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledASMLGuildOfficerTest(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- ASML Raid Master Test Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-asml-test-rm")
        info.checked = function() return HTConfig:GetIsEnabledASMLRaidMasterTest(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledASMLRaidMasterTest()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-asml-test-rm-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledASMLRaidMasterTest(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- ASML Designated ML Test Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-asml-test-dml")
        info.checked = function() return HTConfig:GetIsEnabledASMLDesignatedMasterLooterTest(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledASMLDesignatedMasterLooterTest()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-asml-test-dml-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledASMLDesignatedMasterLooterTest(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

      end

      -- ======================================================================
      -- == Render the level 3 submenu for: Debug Filters =====================
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuDebugFilters" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-debugfilters-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Set Debug Filter for Buff Announcer output.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-debugfilter-ba")
        info.func = function()
                      HTConfig:SetDebugFilter(L["ldb-menu-item-debugfilter-ba-data-val"], L["ldb-menu-item-debugfilter-ba-data-val"])
                      HTBrokerSupport:DisplayValue(L["ldb-menu-item-debugfilter-ba-txt-name"], L["ldb-menu-item-debugfilter-ba-data-val"])
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Set Debug Filter for Buff Checker output.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-debugfilter-bc")
        info.func = function()
                      HTConfig:SetDebugFilter(L["ldb-menu-item-debugfilter-bc-data-val"], L["ldb-menu-item-debugfilter-bc-data-val"])
                      HTBrokerSupport:DisplayValue(L["ldb-menu-item-debugfilter-bc-txt-name"], L["ldb-menu-item-debugfilter-bc-data-val"])
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Set Debug Filter for Busy Responder output.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-debugfilter-br")
        info.func = function()
                      HTConfig:SetDebugFilter(L["ldb-menu-item-debugfilter-br-data-val"], L["ldb-menu-item-debugfilter-br-data-val"])
                      HTBrokerSupport:DisplayValue(L["ldb-menu-item-debugfilter-br-txt-name"], L["ldb-menu-item-debugfilter-br-data-val"])
                    end
        UIDropDownMenu_AddButton(info, level)

        -- Set Debug Filter for Recruitment output.
        HTBrokerSupport:AddMenuPushButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-debugfilter-rms")
        info.func = function()
                      HTConfig:SetDebugFilter(L["ldb-menu-item-debugfilter-rms-data-val"], L["ldb-menu-item-debugfilter-rms-data-val"])
                      HTBrokerSupport:DisplayValue(L["ldb-menu-item-debugfilter-rms-txt-name"], L["ldb-menu-item-debugfilter-rms-data-val"])
                    end
        UIDropDownMenu_AddButton(info, level)

      end

      -- ======================================================================
      -- == Render the level 3 submenu for: Text Label Options ================
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuTextLabelOptions" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-textlabel-options-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Set Text Label Option for Busy Responder Status Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-textlabel-options-br-msgs")
        info.checked = function() return HTConfig:GetIsEnabledTextLabelContentBRMessageNum(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledTextLabelContentBRMessageNum()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-textlabel-options-br-msgs-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledTextLabelContentBRMessageNum(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Set Text Label Option for Guild Stats Members Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-textlabel-options-gs-members")
        info.checked = function() return HTConfig:GetIsEnabledTextLabelContentGStatsMembers(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledTextLabelContentGStatsMembers()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-textlabel-options-gs-members-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledTextLabelContentGStatsMembers(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Set Text Label Option for Guild Stats Raiders Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-textlabel-options-gs-raiders")
        info.checked = function() return HTConfig:GetIsEnabledTextLabelContentGStatsRaiders(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledTextLabelContentGStatsRaiders()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-textlabel-options-gs-raiders-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledTextLabelContentGStatsRaiders(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Set Text Label Option for Guild Stats Officers Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-textlabel-options-gs-officers")
        info.checked = function() return HTConfig:GetIsEnabledTextLabelContentGStatsOfficers(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledTextLabelContentGStatsOfficers()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-textlabel-options-gs-officers-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledTextLabelContentGStatsOfficers(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Set Text Label Option for Guild Stats Characters Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-textlabel-options-gs-characters")
        info.checked = function() return HTConfig:GetIsEnabledTextLabelContentGStatsCharacters(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledTextLabelContentGStatsCharacters()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-textlabel-options-gs-characters-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledTextLabelContentGStatsCharacters(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Set Text Label Option for Guild Play Mode Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-textlabel-options-playmode")
        info.checked = function() return HTConfig:GetIsEnabledTextLabelContentGuildPlayMode(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledTextLabelContentGuildPlayMode()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-textlabel-options-playmode-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledTextLabelContentGuildPlayMode(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Set Text Label Option for Combat Status Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-textlabel-options-combat-status")
        info.checked = function() return HTConfig:GetIsEnabledTextLabelContentCombatStatus(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledTextLabelContentCombatStatus()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-textlabel-options-combat-status-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledTextLabelContentCombatStatus(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Set Text Label Option for Show Minimised Toggle.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-textlabel-options-show-minimised")
        info.checked = function() return HTConfig:GetIsEnabledTextLabelContentShowMinimised(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledTextLabelContentShowMinimised()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-textlabel-options-show-minimised-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledTextLabelContentShowMinimised(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

      end

      -- ======================================================================
      -- == Render the level 3 submenu for: Tooltip Options ===================
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuTooltipOptions" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-tooltip-options-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Set Tooltip Option for Guild Information.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-tooltip-options-guild-info")
        info.checked = function() return HTConfig:GetIsEnabledTooltipContentGuildInformation(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledTooltipContentGuildInformation()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-tooltip-options-guild-info-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledTooltipContentGuildInformation(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Set Tooltip Option for Guild Statistics.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-tooltip-options-guild-stats")
        info.checked = function() return HTConfig:GetIsEnabledTooltipContentGuildStatistics(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledTooltipContentGuildStatistics()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-tooltip-options-guild-stats-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledTooltipContentGuildStatistics(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Set Tooltip Option for Guild Message of the Day.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-tooltip-options-gmotd")
        info.checked = function() return HTConfig:GetIsEnabledTooltipContentGMOTD(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledTooltipContentGMOTD()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-tooltip-options-gmotd-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledTooltipContentGMOTD(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Set Tooltip Option for Officer List.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-tooltip-options-officer-list")
        info.checked = function() return HTConfig:GetIsEnabledTooltipContentOfficerList(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledTooltipContentOfficerList()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-tooltip-options-officer-list-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledTooltipContentOfficerList(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)

        -- Set Tooltip Option for Tool Status Info.
        HTBrokerSupport:AddMenuToggleButton()
        HTBrokerSupport:AddMenuButtonText("ldb-menu-item-tooltip-options-tool-status-info")
        info.checked = function() return HTConfig:GetIsEnabledTooltipContentToolStatusInfo(nil) end
        info.func    = function()
                         HTConfig:ToggleIsEnabledTooltipContentToolStatusInfo()
                         HTBrokerSupport:DisplayStatus(L["ldb-menu-item-tooltip-options-tool-status-info-txt-name"], HTBrokerSupport:IsEnabledText(HTConfig:GetIsEnabledTooltipContentToolStatusInfo(nil)))
                       end
        UIDropDownMenu_AddButton(info, level)
      end

    -- ========================================================================
    -- == Level 4 - Submenus linked from 3rd level menus.                    ==
    -- ========================================================================
    elseif level == 4 then

      -- ======================================================================
      -- == Render the level 4 submenu for: List Associates - Death Knights ===
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuListAssociatesGuildDeathKnights" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-la-death-knights-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Add buttons for all characters of this class.
        local characterList = {}

        for index = 1, (GetNumGuildMembers(true)) do
          local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);
          if string.lower(charClass) == string.lower(L["all-data-classname-deathknight"]) then
            table.insert(characterList, HTData:RemoveRealm(charName))
          end
        end

        table.sort(characterList)

        for i,v in ipairs(characterList) do
          -- Class Specific Selection Button
          HTBrokerSupport:AddMenuPushButton()
          HTBrokerSupport:AddMenuButtonExplicitText(v, v, L["ldb-menu-item-la-character-select-tip-desc"])
          info.func = function() HalonaTools:ListAssociates(v) end
          UIDropDownMenu_AddButton(info, level)
        end

      end

      -- ======================================================================
      -- == Render the level 4 submenu for: List Associates - Druids ==========
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuListAssociatesGuildDruids" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-la-druids-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Add buttons for all characters of this class.
        local characterList = {}

        for index = 1, (GetNumGuildMembers(true)) do
          local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);
          if string.lower(charClass) == string.lower(L["all-data-classname-druid"]) then
            table.insert(characterList, HTData:RemoveRealm(charName))
          end
        end

        table.sort(characterList)

        for i,v in ipairs(characterList) do
          -- Class Specific Selection Button
          HTBrokerSupport:AddMenuPushButton()
          HTBrokerSupport:AddMenuButtonExplicitText(v, v, L["ldb-menu-item-la-character-select-tip-desc"])
          info.func = function() HalonaTools:ListAssociates(v) end
          UIDropDownMenu_AddButton(info, level)
        end

      end

      -- ======================================================================
      -- == Render the level 4 submenu for: List Associates - Mages ===========
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuListAssociatesGuildMages" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-la-mages-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Add buttons for all characters of this class.
        local characterList = {}

        for index = 1, (GetNumGuildMembers(true)) do
          local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);
          if string.lower(charClass) == string.lower(L["all-data-classname-mage"]) then
            table.insert(characterList, HTData:RemoveRealm(charName))
          end
        end

        table.sort(characterList)

        for i,v in ipairs(characterList) do
          -- Class Specific Selection Button
          HTBrokerSupport:AddMenuPushButton()
          HTBrokerSupport:AddMenuButtonExplicitText(v, v, L["ldb-menu-item-la-character-select-tip-desc"])
          info.func = function() HalonaTools:ListAssociates(v) end
          UIDropDownMenu_AddButton(info, level)
        end

      end

      -- ======================================================================
      -- == Render the level 4 submenu for: List Associates - Monks ===========
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuListAssociatesGuildMonks" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-la-monks-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Add buttons for all characters of this class.
        local characterList = {}

        for index = 1, (GetNumGuildMembers(true)) do
          local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);
          if string.lower(charClass) == string.lower(L["all-data-classname-monk"]) then
            table.insert(characterList, HTData:RemoveRealm(charName))
          end
        end

        table.sort(characterList)

        for i,v in ipairs(characterList) do
          -- Class Specific Selection Button
          HTBrokerSupport:AddMenuPushButton()
          HTBrokerSupport:AddMenuButtonExplicitText(v, v, L["ldb-menu-item-la-character-select-tip-desc"])
          info.func = function() HalonaTools:ListAssociates(v) end
          UIDropDownMenu_AddButton(info, level)
        end

      end

      -- ======================================================================
      -- == Render the level 4 submenu for: List Associates - Hunters =========
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuListAssociatesGuildHunters" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-la-hunters-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Add buttons for all characters of this class.
        local characterList = {}

        for index = 1, (GetNumGuildMembers(true)) do
          local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);
          if string.lower(charClass) == string.lower(L["all-data-classname-hunter"]) then
            table.insert(characterList, HTData:RemoveRealm(charName))
          end
        end

        table.sort(characterList)

        for i,v in ipairs(characterList) do
          -- Class Specific Selection Button
          HTBrokerSupport:AddMenuPushButton()
          HTBrokerSupport:AddMenuButtonExplicitText(v, v, L["ldb-menu-item-la-character-select-tip-desc"])
          info.func = function() HalonaTools:ListAssociates(v) end
          UIDropDownMenu_AddButton(info, level)
        end

      end

      -- ======================================================================
      -- == Render the level 4 submenu for: List Associates - Paladins ========
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuListAssociatesGuildPaladins" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-la-paladins-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Add buttons for all characters of this class.
        local characterList = {}

        for index = 1, (GetNumGuildMembers(true)) do
          local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);
          if string.lower(charClass) == string.lower(L["all-data-classname-paladin"]) then
            table.insert(characterList, HTData:RemoveRealm(charName))
          end
        end

        table.sort(characterList)

        for i,v in ipairs(characterList) do
          -- Class Specific Selection Button
          HTBrokerSupport:AddMenuPushButton()
          HTBrokerSupport:AddMenuButtonExplicitText(v, v, L["ldb-menu-item-la-character-select-tip-desc"])
          info.func = function() HalonaTools:ListAssociates(v) end
          UIDropDownMenu_AddButton(info, level)
        end

      end

      -- ======================================================================
      -- == Render the level 4 submenu for: List Associates - Priests =========
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuListAssociatesGuildPriests" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-la-priests-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Add buttons for all characters of this class.
        local characterList = {}

        for index = 1, (GetNumGuildMembers(true)) do
          local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);
          if string.lower(charClass) == string.lower(L["all-data-classname-priest"]) then
            table.insert(characterList, HTData:RemoveRealm(charName))
          end
        end

        table.sort(characterList)

        for i,v in ipairs(characterList) do
          -- Class Specific Selection Button
          HTBrokerSupport:AddMenuPushButton()
          HTBrokerSupport:AddMenuButtonExplicitText(v, v, L["ldb-menu-item-la-character-select-tip-desc"])
          info.func = function() HalonaTools:ListAssociates(v) end
          UIDropDownMenu_AddButton(info, level)
        end

      end

      -- ======================================================================
      -- == Render the level 4 submenu for: List Associates - Rogues ==========
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuListAssociatesGuildRogues" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-la-rogues-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Add buttons for all characters of this class.
        local characterList = {}

        for index = 1, (GetNumGuildMembers(true)) do
          local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);
          if string.lower(charClass) == string.lower(L["all-data-classname-rogue"]) then
            table.insert(characterList, HTData:RemoveRealm(charName))
          end
        end

        table.sort(characterList)

        for i,v in ipairs(characterList) do
          -- Class Specific Selection Button
          HTBrokerSupport:AddMenuPushButton()
          HTBrokerSupport:AddMenuButtonExplicitText(v, v, L["ldb-menu-item-la-character-select-tip-desc"])
          info.func = function() HalonaTools:ListAssociates(v) end
          UIDropDownMenu_AddButton(info, level)
        end

      end

      -- ======================================================================
      -- == Render the level 4 submenu for: List Associates - Shaman ==========
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuListAssociatesGuildShaman" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-la-shaman-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Add buttons for all characters of this class.
        local characterList = {}

        for index = 1, (GetNumGuildMembers(true)) do
          local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);
          if string.lower(charClass) == string.lower(L["all-data-classname-shaman"]) then
            table.insert(characterList, HTData:RemoveRealm(charName))
          end
        end

        table.sort(characterList)

        for i,v in ipairs(characterList) do
          -- Class Specific Selection Button
          HTBrokerSupport:AddMenuPushButton()
          HTBrokerSupport:AddMenuButtonExplicitText(v, v, L["ldb-menu-item-la-character-select-tip-desc"])
          info.func = function() HalonaTools:ListAssociates(v) end
          UIDropDownMenu_AddButton(info, level)
        end

      end

      -- ======================================================================
      -- == Render the level 4 submenu for: List Associates - Warlocks ========
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuListAssociatesGuildWarlocks" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-la-warlocks-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Add buttons for all characters of this class.
        local characterList = {}

        for index = 1, (GetNumGuildMembers(true)) do
          local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);
          if string.lower(charClass) == string.lower(L["all-data-classname-warlock"]) then
            table.insert(characterList, HTData:RemoveRealm(charName))
          end
        end

        table.sort(characterList)

        for i,v in ipairs(characterList) do
          -- Class Specific Selection Button
          HTBrokerSupport:AddMenuPushButton()
          HTBrokerSupport:AddMenuButtonExplicitText(v, v, L["ldb-menu-item-la-character-select-tip-desc"])
          info.func = function() HalonaTools:ListAssociates(v) end
          UIDropDownMenu_AddButton(info, level)
        end

      end

      -- ======================================================================
      -- == Render the level 4 submenu for: List Associates - Warriors ========
      -- ======================================================================
      if UIDROPDOWNMENU_MENU_VALUE == "SubmenuListAssociatesGuildWarriors" then

        -- General Menu Defaults.
        HTBrokerSupport:AddMenuDefaultConfig()

        -- Create the title of the menu
        HTBrokerSupport:AddMenuTitle(L["ldb-menu-la-warriors-name"])
        UIDropDownMenu_AddButton(info, level)

        -- Empty Menu Item.
        HTBrokerSupport:AddMenuDummyButton()
        UIDropDownMenu_AddButton(info, level)

        -- Add buttons for all characters of this class.
        local characterList = {}

        for index = 1, (GetNumGuildMembers(true)) do
          local charName, charRank, charRankIndex, charLevel, charClass, charZone, charNote, charOfficernote, charOnline, charStatus = GetGuildRosterInfo(index);
          if string.lower(charClass) == string.lower(L["all-data-classname-warrior"]) then
            table.insert(characterList, HTData:RemoveRealm(charName))
          end
        end

        table.sort(characterList)

        for i,v in ipairs(characterList) do
          -- Class Specific Selection Button
          HTBrokerSupport:AddMenuPushButton()
          HTBrokerSupport:AddMenuButtonExplicitText(v, v, L["ldb-menu-item-la-character-select-tip-desc"])
          info.func = function() HalonaTools:ListAssociates(v) end
          UIDropDownMenu_AddButton(info, level)
        end

      end

    end
  end
end

-- ============================================================================
-- == AddMenuDefaultConfig()                                                 ==
-- ==                                                                        ==
-- == This function adds the default meta-data required by all menu items.   ==
-- ============================================================================

function HTBrokerSupport:AddMenuDefaultConfig()

  info.keepShownOnClick = 1

end

-- ============================================================================
-- == AddMenuTitle(title)                                                    ==
-- ==                                                                        ==
-- == This function adds the meta-data required for a new menu title.        ==
-- ============================================================================

function HTBrokerSupport:AddMenuTitle(title)

  -- Create the title of the menu
  info.isTitle      = 1
  info.text         = title
  info.notCheckable = 1

end

-- ============================================================================
-- == AddMenuDummyButton()                                                   ==
-- ==                                                                        ==
-- == This function adds the meta-data required for a new dummy button.      ==
-- ============================================================================

function HTBrokerSupport:AddMenuDummyButton()

  -- Empty Menu Item.
  info.text         = " "
  info.checked      = nil
  info.notCheckable = 1
  info.hasArrow     = nil
  info.value        = nil
  info.disabled     = 1

end

-- ============================================================================
-- == AddMenuSubMenu()                                                       ==
-- ==                                                                        ==
-- == This function adds the meta-data required for a new sub-menu.          ==
-- ============================================================================

function HTBrokerSupport:AddMenuSubMenu(name)

  info.isTitle      = nil
  info.disabled     = nil
  info.func         = self.UncheckHack
  info.hasArrow     = 1
  info.value        = name
  info.notCheckable = 1

end

-- ============================================================================
-- == AddMenuPushButton()                                                    ==
-- ==                                                                        ==
-- == This function adds the meta-data required for a new push button.       ==
-- ============================================================================

function HTBrokerSupport:AddMenuPushButton()

  info.isTitle      = nil
  info.disabled     = nil
  info.hasArrow     = nil
  info.value        = nil
  info.notCheckable = 1

end

-- ============================================================================
-- == AddMenuToggleButton()                                                  ==
-- ==                                                                        ==
-- == This function adds the meta-data required for a new toggle button.     ==
-- ============================================================================

function HTBrokerSupport:AddMenuToggleButton()

  info.isTitle      = nil
  info.disabled     = nil
  info.hasArrow     = nil
  info.notCheckable = nil

end

-- ============================================================================
-- == AddMenuButtonText(referenceStub)                                       ==
-- ==                                                                        ==
-- == This function adds menu button text fields to the 'info' table for the ==
-- == supplied referenceStub value.                                          ==
-- ============================================================================

function HTBrokerSupport:AddMenuButtonText(localeString)

  local txtName = localeString .. "-txt-name"
  local tipName = localeString .. "-tip-name"
  local tipDesc = localeString .. "-tip-desc"

  info.text         = L[txtName]
  info.tooltipTitle = L[tipName]
  info.tooltipText  = L[tipDesc]

end

-- ============================================================================
-- == AddMenuButtonExplicitText(text, tipTitle, tipText)                     ==
-- ==                                                                        ==
-- == This function adds menu button text fields to the 'info' table for the ==
-- == supplied values.                                                       ==
-- ============================================================================

function HTBrokerSupport:AddMenuButtonExplicitText(text, tipTitle, tipText)

  info.text         = text
  info.tooltipTitle = tipTitle
  info.tooltipText  = tipText

end

-- ============================================================================
-- == IsEnabledText(value)                                                   ==
-- ==                                                                        ==
-- == This function determines whether the supplied value is recognised as   ==
-- == meaning "is enabled" and returns 'On' if enabled, 'Off' otherwise.     ==
-- ============================================================================

function HTBrokerSupport:IsEnabledText(value)

  if value == true then
    return "|cff00ff00On|r"
  else
    return "|cffff0000Off|r"
  end

end

-- ============================================================================
-- == DisplayStatus(featureName, isEnabledText)                              ==
-- ==                                                                        ==
-- == This function enables the status of a feature to be displayed via the  ==
-- == standard chat-window system with appropriate colourisation.            ==
-- ============================================================================

function HTBrokerSupport:DisplayStatus(featureName, isEnabledText)

  local message = L["cfg-display-data-title"] .. featureName .. ": " .. isEnabledText

  HTConsole:SendChatMessage(message, "CHAT", nil, nil)

end

-- ============================================================================
-- == DisplayValue(featureName, value)                                       ==
-- ==                                                                        ==
-- == This function enables the value of a feature to be displayed via the   ==
-- == standard chat-window system.                                           ==
-- ============================================================================

function HTBrokerSupport:DisplayValue(featureName, value)

  local message = L["cfg-display-data-title"] .. featureName .. ": |cffffff00" .. value .. "|r"

  HTConsole:SendChatMessage(message, "CHAT", nil, nil)

end

-- ============================================================================
-- == GetTooltipOfficerList()                                                ==
-- ==                                                                        ==
-- == This function generates the officer list required for display in the   ==
-- == LDB tooltip.                                                           ==
-- ============================================================================

function HTBrokerSupport:GetTooltipOfficerList()

  local rawList            = HTCommon:GetGuildOfficerList()
  local onlineOfficers     = {}
  local offlineOfficers    = {}
  local tooltipOfficerList = {}

  for i,v in ipairs(rawList) do

    local characterName = HTCommon:FindPlayerByCharacterName(v)
    local testName      = characterName -- Needed because string.find alters string.

    local simpleCharacterName = HTCommon:GetSimpleCharacterName(characterName)
    local offlineTime         = HTCommon:GetTimeOffline(v, false)

    if string.match(testName, L["ldb-tooltips-officers-offline-query"]) then
      table.insert(offlineOfficers, "|cff990000" .. characterName .. "|r" .. L["all-text-separator"] .. "|cff666666" .. offlineTime .. "|r")
    else
      table.insert(onlineOfficers, "|cff00ff00" .. characterName .. "|r" .. L["all-text-separator"] .. L["all-data-unset"])
    end

    table.sort(onlineOfficers)
    table.sort(offlineOfficers)

  end

  tooltipOfficerList = HTData:JoinTables(onlineOfficers, offlineOfficers)

  return tooltipOfficerList

end
